//=============================================================================
// UTDMT.
//
// This mod was made to make a talking Titan for the level
// Titania.  There should be a readme file with usage 
// information.  See this file for implementation details.
//
// There was an Unreal version of this mod originally, called
// DeathMatchTitan.
//
/* Changes in this version:

- : Not done.
* : Done.
x : Prohibitively difficult or a dumb idea.

* Respawning.
  Done by creating a UTDMTStartPoint where the titan started at..
* Doesn't pick up, use, or drop weapons.
  Done with the UTDMTMutator.
* Each type of taunt: count up to 50.  New default taunts added.
  Done to Squacky's satisfaction.
* Uses UT-type messages (shows face in upper left of window).
  All that was required was to add a face texture to the titan's PRI.
* Beep along with taunt message.
* Server admin can turn on/off titan auto-messaging.
  Done, though with 'memory.' See next.
* Server admin can send messages through the titan.
  If only a titan with a particular name is supposed 
  to say this message, the format is this:
  [set utdmt msg #name#Message]
  If this format is done incorrectly the entire message
  will be said by every titan, so get it right.
* Messages configurable by file, instead of by level.
  Kind of: configurable by file instead of default, but level always takes priority.
* Identify info.
* Titans can't have same name: if duplicate found, titans start naming themselves
  Titan1, Titan2, etc.
* Add fictitious ping.
* Shouldn't attack anything before the game starts.
* Flag which makes titans not hate one another.
* Should be able to be on teams.
* Team damage should work for taking damage.
* Delay after message event--more realistic, since it takes real people some time to type
  stuff in.
* Titans don't dissappear when the match ends.
  Not only that, but they dance a bit, and sometimes say DanceMsgs.
* SawWithPawn should make up a name if not given one.
* If not a team game, overrides level-set team number.
* Death message overrides wait messages.
* Configurable wait message probability.
* Shouldn't take orders: should send negatory message to teammates only!

*** These aren't a real high priority.
- Team damage should work for delivering damage (slap, rocks).
- Custom death messages (crushed by rock, etc).

x Taunt player with the lowest score.
  Not hard to add, but often in network games there are multiple people with the lowest 
  score because people come and go a lot.
x Roar/whatever along with messages.
  This isn't practical since Titans make a lot of noise as it is, and especially
  in situations we use to trigger messages.

*/

// Notes:
// - If the Titan is very busy you may find that he won't say a Victory message.
//		This is probably because he is simply too busy and skips the VictoryDance state,
//		which is where he says the VictoryMsgs.
//
//=============================================================================
class UTDMT expands StoneTitan 
	config(UTDMT);

// Import the face texture.
#exec TEXTURE IMPORT NAME=TitanFace FILE=Textures\TitanFace.pcx GROUP=Face LODSET=2

// This is the name he will have in deathmatch/botmatch games.
// Shouldn't be longer than 20.
var() String MyName;

// If the user specified name is being used, this is the 
// name we use instead, or this name + # if there are multiple titans being
// used.  Example: if you add two titans to a level and don't set their names
// they will become "DefaultName1" and "DefaultName2".
// Shouldn't be longer than 20.
var config string DefaultName;

// Which team this titan is on.  Configurable so CTF map makers can use them
// to guard flags, etc.  No team is 255, Red is 0, Blue is 1, etc.
var() byte Team;

// Whether or not to respawn.  Keep in mind they WILL telefrag people
var() config bool bRespawn;

// How much time between his death and a respawn.
var() config float RespawnDelay;

// If false, then we won't attack members of our own class.
var() config bool bHateOurOwn;

// Allows you to turn his ability to "say" on or off.
var() bool bNoTalking;

// If true, then when we are triggered we will try to say one of our TriggerMsgs.
var() bool bEnableTriggering;

// Messages are delayed to simulate the time it takes people to type in messages.
// How much to delay messages by.  Formula is: delay = base + spc * Len(msg)
var() float fDelayBaseSecs;
var() float fDelaySecsPerChar;

// The 'frequency' of wait messages.  Think around .2 or less.
var() float fWaitMsgProbability;

// *** Message arrays.
// C++ Unreal script is not...no obvious way to avoid having a magic
// number in these array declarations.  

// Said when triggered.
var(DMTMsgs) config String TriggerMsgs[31];
// Said when dying.
var(DMTMsgs) config String DyingMsgs[21];
// Said when we kill someone.
var(DMTMsgs) config String VictoryMsgs[51];
// Say when we are hurt: more specifically, when we show a hurt animation.  Usually
// only happens when we are about to die.
var(DMTMsgs) config String HurtMsgs[31];
// Said when we are not doing anything else (like fighting).
var(DMTMsgs) config String WaitMsgs[41];
// Said when someone on our own team gives us an order (like guard the flag, etc).
// We do not take orders.
var(DMTMsgs) config String IgnoreOrderMsgs[20];
// Used when there is a %p in the message but we cannot determine the player name to 
// use.
var(DMTMsgs) config String InsultingNames[10];
// At the end of the game we dance around a bit, and occasionally say these messages.
var(DMTMsgs) config String DanceMsgs[10];

// Said a the end of the game when we are:
// alive.
var() config String EndGameAliveMsg;
// dead.
var() config String EndGameDeadMsg;

// *** 

// Use "set utdmt msg [blah]" to make the titan say "blah"--see instructions.
var string msg;

// *** Internal variables.

// The point we started at, and where we will respawn to if we must 
// respawn.
var UTDMTStartPoint startPoint;

// The class that delays messages for us.
var UTDMTMsgDelayer msgDelayer;

// *** Internal Variables used to simulate ping.

// The value to base this titan's ping around.
var int iPingBase;

// Max +- from PingBase ping should fluxuate.
var int iPingMaxVariation;

// Time we need to wait to refresh ping.
var int iPingRefreshTime;

// Base amount of time between Ping refreshes.
var int iPingRefreshBase;

// Max +- ping refresh time should fluxuate.
var int iPingRefreshMaxVariation;

// Time since last "ping" update.
var float fPingTimer;

////////////////////////////////////////////////////////////

// We check here to see if the admin set msg to anything.
// If so, we say it.
function tick(float fDeltaT)
{
	local String strTitanName;
	local int iNameEnd;

	// *** Ping
	
	fPingTimer += fDeltaT;

	// 2.0 taken from PRI as being how often it rechecks ping.
	if(fPingTimer > iPingRefreshTime)
	{
		fPingTimer = 0;

		CalculatePing();
	}

	// *** Check for messages.

	if(msg != "")
	{
		// If only a titan with a particular name is supposed 
		// to say this message, the format is this:
		// set utdmt msg #name#Message
		if(Left(msg, 1) == "#")
		{
			strTitanName = Right(msg, Len(msg) - 1);
			iNameEnd = InStr(strTitanName, "#");
			if(iNameEnd != -1)
			{
				// Get the name of the titan to say this message.
				strTitanName = Left(strTitanName, iNameEnd);

				// If it isn't us, don't say it. (case-insensitive player name comparison)
				if(!(strTitanName ~= PlayerReplicationInfo.PlayerName))
				{
					//Log(class$":tick : We are '"$MyName$"', message '"$msg$"' not for me, for titan '"$strTitanName);
					return;
				}

				// If it is, strip the titan name from the message
				// and go ahead and say it.
				msg = Right(msg, Len(msg) - Len(strTitanName) - 2);
			}
		}
		
		SayCore(msg);
		msg = "";
	}
}

////////////////////////////////////////////////////////////

// This function is called before Spawned is.
simulated function PreBeginPlay()
{
	local UTDMTMutator dmtMutator;
	local bool bMutatorSpawned;
	local int iNameNumber;
	local string MyNewName;

//*** Mega important stuff:
	// This is what puts him in the Deathmatch scores list, and allows him to win.
	// Pawn.PreBeginPlay sets up the PlayerReplicationInfoClass for this object if
	// bIsPlayer is true, so we need to be sure to set it BEFORE calling
	// the super.PreBeginPlay().
	bIsPlayer = true;
	Super.PreBeginPlay();

// *** Set start point

	// Initialize our start point.

		startPoint = spawn(class'UTDMTStartPoint', self, , 
			Location, Rotation);
		if(startPoint == None)
		{
			Log(class$":PreBeginPlay : Couldn't spawn startPoint.");
		}
/*		else
		{
			log (Self$" have startPoint with Team "$startPoint.Team);
			StartPoint.Team = PlayerReplicationInfo.Team;
			Team = PlayerReplicationInfo.Team;
		}
*/

// *** Texture

	// Set the talktexture
	if (PlayerReplicationInfo != None)
	{
		PlayerReplicationInfo.TalkTexture = Texture'UTDMT.Face.TitanFace';
		if(PlayerReplicationInfo.TalkTexture == None)
		{
			Log(class$":PreBeginPlay : Couldn't set TalkTexture.");
		}
	}
// *** Create mutator

	// See if a UTDMTMutator exists.
	bMutatorSpawned = false;
	foreach AllActors(class'UTDMTMutator', dmtMutator)
	{
		bMutatorSpawned = true;
	}

	// If it hasn't been made yet, make one and add it to the level's
	// list.

	if(!bMutatorSpawned)
	{
		Log(class$":PreBeginPlay : Creating a UTDMTMutator.");
		level.game.BaseMutator.AddMutator(Spawn(class'UTDMTMutator'));
	}

// *** Set name
	iNameNumber = 0;
	MyNewName = MyName;
	While(IsPlayerNameBeingUsed(MyNewName))
	{
		// Generate a new name.
		iNameNumber++;
		MyNewName = DefaultName$iNameNumber;
	}

	if(MyName != MyNewName)
	{
		Log(class$":PreBeginPlay : Name '"$MyName$"' in use, using '"$MyNewName$"' instead.");
		MyName = MyNewName;
	}

	// Ths sets his name.  I think this has to happen AFTER Super.PreBeginPlay().
	PlayerReplicationInfo.PlayerName = MyName;

// *** Ping

	// Set base ping to somewhere between 50 and 250;
	iPingBase = 150 + Rand(200) - 100;

	// Seed ping recalculation (random hundredth's of seconds up to the refresh time).
	fPingTimer = iPingRefreshTime * FRand();

	CalculatePing();

// *** Team

	if(Level.Game.IsA('TeamGamePlus'))
		PlayerReplicationInfo.Team = Team;
	else
		PlayerReplicationInfo.Team = 255;

// *** Inventory

	SetInventory();

// *** Message delayer.

	if(msgDelayer == None)
	{
		msgDelayer = Spawn(class'UTDMTMsgDelayer');
		if(msgDelayer == None)
		{
			Log(class$": Error spawning a UTDMTMsgDelayer.");
		}
	}

	msgDelayer.dmtOwner = self;
	msgDelayer.fBaseSecs = fDelayBaseSecs;
    msgDelayer.fSecsPerChar = fDelaySecsPerChar;
	msgDelayer.bDisabled = false;
}

function bool IsPlayerNameBeingUsed(String strName)
{
	local PlayerReplicationInfo otherPRI;

	ForEach AllActors(class'PlayerReplicationInfo', otherPRI)
	{
		// Check to see if we have the same name as someone else.
		if(otherPRI != PlayerReplicationInfo &&
			otherPRI.PlayerName ~= strName)
		{
			return true;
		}
	}

	return false;
}

function CalculatePing()
{
	if (Health <= 0) return; //no need to mock with unexistent data
	// Make our displayed ping our base give or take x.
	PlayerReplicationInfo.Ping = iPingBase + Rand(iPingMaxVariation * 2) - iPingMaxVariation;
	// Calculate new refresh time.
	iPingRefreshTime = iPingRefreshBase + Rand(iPingRefreshMaxVariation * 2) - iPingRefreshMaxVariation;
}

//////////////////////////////////
// Functions for sending messages
// in multiplayer games.
//////////////////////////////////

// Send a message to all players.
// Essense copied from PlayerPawn::Say
// This function should generally not be called from the script as it doesn't delay the
// messages.
function SayCore( string Msg )
{
	local Pawn P;

	// Check to see if the message string is empty.

	if(0 == Len(Msg))
		return;

	for( P=Level.PawnList; P!=None; P=P.nextPawn )
		if( P.bIsPlayer || P.IsA('MessagingSpectator') )
		{
			if(Left(msg, 1) == "@")
			{
				// If not on our team skip to next pawn.
				if(P.PlayerReplicationInfo == None || 
					P.PlayerReplicationInfo.Team != PlayerReplicationInfo.Team)
				{
					continue;
				}

				// Trim the @ off.
				msg = Right(msg, Len(msg) - 1);
			}

			if ( (Level.Game != None) && (Level.Game.MessageMutator != None) )
			{
				if ( Level.Game.MessageMutator.MutatorTeamMessage(Self, P, PlayerReplicationInfo, Msg, 'Say', true) )
					P.TeamMessage( PlayerReplicationInfo, Msg, 'Say', true );
			} else
				P.TeamMessage( PlayerReplicationInfo, Msg, 'Say', true );
		}
	return;
}

// Call this function from within the class to make the Titan say a message.
function Say( String Msg )
{
	// Check to see if the message string is empty.
	if(0 == Len(Msg))
		return;

	// This is where bNoTalking does its work.
	if(bNoTalking)
		return;

	// Delay the message.
	if(msgDelayer != None)
		msgDelayer.DelayMsg(msg);
}

// This "Say" takes a string and another string.  If the string contains a token then 
// it will be replaced by the filler string.
function SayWithString(String msg, String filler)
{
	local int tokenPos;
	local String strleft;
	local String strright;
	local int strlen;
	local String outputMsg;
	local String tokenType;

	//Log("SayWithString(" $ msg $ ", " $ filler $ ") called");

	// Check this here for efficiency.
	if(bNoTalking)
		return;

	// Copy the message into a local string for manipulation.
	outputMsg = Msg;

	// The filler absolutely cannot have a % in it!
	if(-1 != InStr(filler, "%"))
	{
		//Log(String(Self) $ " : " $ filler $ " contains a %, thus it cannot be 'Say'ed.");
		return;
	}

	// This is a special case: if the filler string is "", only print the output if 
	// the msg string has no %.
	if(0 == Len(filler))
	{
		if(InStr(outputMsg, "%") >= 0)
		{
			//Log("DMT: no string but token found...output cancelled.");
			return;
		}
		Say(outputMsg);
	}

	// Scan the message for a token.
	tokenPos = InStr(outputMsg, "%");
	while(-1 != tokenPos)
	{
		//Log("Beginning switch...outputMsg = " $ outputMsg);
		//Log("tokenPos = " $ String(tokenPos));
		tokenType =  Mid(outputMsg, tokenPos + 1, 1);

		// p for Pawn: typically the instigator of the event, but can be anything.
		if(tokenType == "p")
		{
			strleft = Left(outputMsg, tokenPos);
			strlen = Len(outputMsg);
			strright = Right(outputMsg, strlen - tokenPos - 2);
			
			outputMsg = strleft $ filler $ strright;
		}
		tokenPos = InStr(outputMsg, "%");
	}

	// Say the modified message.
	Say(outputMsg);
}

// This version of Say takes a reference to a Pawn as its second argument.
// For added flexibility, if the Pawn has no name one is picked from a list.
function SayWithPawn(String msg, Pawn aPawn)
{
	local string pawnName;
	
	//Log("DMT: SayWithPawn called, msg = " $Msg$ ", pawn is a " $String(aPawn));
	
	if(aPawn == None)
	{
		SayWithString(msg, InsultingNames[Rand(ArrayCount(InsultingNames))]);
		return;
	}
	if (aPawn.PlayerReplicationInfo != None)
		pawnName = aPawn.PlayerReplicationInfo.PlayerName;
	if(0 == len(pawnName))
	{
		SayWithString(msg, InsultingNames[Rand(ArrayCount(InsultingNames))]);
		return;
	}

	SayWithString(msg, pawnName);
}

// ***
// I wrote this function to represent what he does when the trap is opened:
// Tries to talk people into coming down.
// ***
function Trigger( actor Other, pawn EventInstigator )
{
	local String instigatorName;

	if(!bEnableTriggering || EventInstigator == Self)
		return;

	//Log("DMT Triggered by a " $ String(EventInstigator));
	if (EventInstigator.PlayerReplicationInfo != None && EventInstigator != Self)
		instigatorName = EventInstigator.PlayerReplicationInfo.PlayerName;
	if(0 == Len(instigatorName))
	{
		//Log("DMT: Trigger instigator has no name.");
		return;
	}
	SayWithString(TriggerMsgs[Rand(ArrayCount(TriggerMsgs))], instigatorName);
}
/*
function damageAttitudeTo(pawn Other)
{
	local eAttitude OldAttitude;

	if ( (Other == Self) || (Other == None) /*|| (FlockPawn(Other) != None) */)
		return;
	if( Other.bIsPlayer ) //change attitude to player
	{ //FIXME - also frenzy or run away against non-players
		if ( (Health < 30) && (Aggressiveness * FRand() > 0.5) )
		{
			AttitudeToPlayer = ATTITUDE_Frenzy;
			Aggressiveness = 1.0;
		}
		else if (AttitudeToPlayer == ATTITUDE_Ignore) AttitudeToPlayer = ATTITUDE_Hate;
		else if (AttitudeToPlayer == ATTITUDE_Threaten) AttitudeToPlayer = ATTITUDE_Hate;
		else if (AttitudeToPlayer == ATTITUDE_Friendly) AttitudeToPlayer = ATTITUDE_Threaten;
	}
	else 
	{
		OldAttitude = AttitudeToCreature(Other);
		if (OldAttitude > ATTITUDE_Ignore )
			return;
		else if ( OldAttitude > ATTITUDE_Frenzy )
		{
			//log(class$" hates "$Other.class);
			Hated = Other;
		}
		if ( AttitudeTo(Other) <= ATTITUDE_Ignore )
		{
			if ( Other.Target != None && Other.Target == Self )
			{
				AttitudeTo(Other) == ATTITUDE_Hate;
				Enemy = Other;
				Hated = Other;
			}
		}
	}
	if ( SetEnemy(Other) )
		NextState('Attacking');
}
*/
function damageAttitudeTo(pawn Other)
{
	local eAttitude OldAttitude;

//Step 1) - Prevent craps - probably needs more

	if ( (Other == Self) || (Other == None) /*|| (FlockPawn(Other) != None)*/ ) //I'll out this back when Biterfish are going to be friendly - next century
		return;
	OldAttitude = AttitudeTo(Other);

	//Step 2) - a TeamGame - ANY 
	if ( Level.Game.bTeamGame )
	{
		if ( OldAttitude > ATTITUDE_Ignore )
			return;
		else if ( OldAttitude <= ATTITUDE_Ignore )
			Hated = Other;
	} //Step 3) - not a TeamGame
	else
		Hated = Other;
	SetEnemy(Other);
}

// This copied directly from ScriptedPawn.  Only modification is the addition of
// a message when he does an animation.
function PlayWaiting()
{
	local float decision;
	local float animspeed;
	//*** 
	local float fTalkRand;
	//***

	decision = FRand();
	animspeed = 0.2 + 0.5 * FRand();

	if (bEndFootStep)
		FootStep();
	
	//*** 
	fTalkRand = FRand();
	if(fTalkRand < fWaitMsgProbability)
		Say(WaitMsgs[Rand(ArrayCount(WaitMsgs))]);
	//*** 

	if ( (AnimSequence == 'TBrea001') && (decision < 0.17) )
	{
		SetAlertness(0.0);
		if (decision < 0.1)
		{
			PlaySound(sound'sniff1Ti', SLOT_Talk);
			LoopAnim('TSnif001', animspeed);
		}
		else if (decision < 0.17)
			LoopAnim('TFist', animspeed);
	}
	else
	{
		SetAlertness(0.3);
		LoopAnim('TBrea001', animspeed);
	}
}

/* Using mutator to do this, as this approach didn't work.
// Add Item to this pawn's inventory. 
// Returns true if successfully added, false if not.
// TITANS CANNOT PICK UP INVENTORY ITEMS.
function bool AddInventory( inventory NewItem )
{
	Log(class$":AddInventory : rejecting a "$NewItem);
	return false;
}
*/

function Died(pawn Killer, name damageType, vector HitLocation)
{
	local string pawnName;

	// Clear out any pain msgs, which are likely to be there, in order to get the
	// death message out promptly.
	msgDelayer.Flush();
	SayWithPawn(DyingMsgs[Rand(ArrayCount(DyingMsgs))], Killer);

	// Be sure to call the base class implementation, or he doesn't 
	// die!  He will eventually get gibbed, though.
	Super.Died(Killer, damageType, HitLocation);
}

// This isn't terribly efficient, but it is reliable.
function Killed(pawn Killer, pawn Other, name damageType)
{
	if ( Other != None && ( Killer != None && Killer == Self ) ) 
	if ( Other.PlayerReplicationInfo != None ) 
		Log(class$": Killed: Me "$PlayerReplicationInfo.PlayerName$", killed "$Other.PlayerReplicationInfo.PlayerName);
	if ( Killer != None && Self == Killer && Other != None) 
		SayWithPawn(VictoryMsgs[Rand(ArrayCount(VictoryMsgs))], Other);
	Super.Killed(Killer, Other, damageType);
}

// This is based on Pawn::RestartPlayer.
function bool RestartTitan( UTDMT aPlayer )
{
	local bool foundStart;

	//?
	//if( bRestartLevel && Level.NetMode!=NM_DedicatedServer && Level.NetMode!=NM_ListenServer )
	//	return true;

	if( startPoint == None )
	{
		log(class$"::RestartPlayer : startPoint = None!");
		return false;
	}

	foundStart = aPlayer.SetLocation(startPoint.Location);
	if( foundStart )
	{
		startPoint.PlayTeleportEffect(aPlayer, true);
		aPlayer.SetRotation(startPoint.Rotation);
		aPlayer.ViewRotation = aPlayer.Rotation;
		aPlayer.Acceleration = vect(0,0,0);
		aPlayer.Velocity = vect(0,0,0);
		aPlayer.Health = aPlayer.Default.Health;
		aPlayer.SetCollision( true, true, true );
		aPlayer.ClientSetLocation( startPoint.Location, startPoint.Rotation );
		aPlayer.bHidden = false;
		aPlayer.DamageScaling = aPlayer.Default.DamageScaling;
		aPlayer.SoundDampening = aPlayer.Default.SoundDampening;
		// ***
		SetInventory();
/*		if (startPoint != None)
			if (TeamGamePlus(Level.Game) != None)
				TeamGamePlus(Level.Game).ChangeTeam(Self,startPoint.Team);
*/
	}
	else
		log(startPoint$" Player start not useable!!!");

	return foundStart;
}

// Net Code Boulders
//Initially they didn't rotate in Clients :(

simulated function SpawnRock()
{
	local Projectile Proj;
	local vector X,Y,Z, projStart;
	GetAxes(Rotation,X,Y,Z);
	
	MakeNoise(1.0);
	if (FRand() < 0.4)
	{
		projStart = Location + CollisionRadius * X + 0.4 * CollisionHeight * Z;
		Proj = spawn(class 'DMT_Boulder1' ,self,'',projStart,AdjustAim(1000, projStart, 400, false, true));
		if( Proj != None )
			Proj.SetPhysics(PHYS_Projectile);
		return;
	}

	projStart = Location + CollisionRadius * X + 0.4 * CollisionHeight * Z;
	Proj = spawn(class 'DMT_BigRock' ,self,'',projStart,AdjustAim(1000, projStart, 400, false, true));
	if( Proj != None )
		Proj.SetPhysics(PHYS_Projectile);

	projStart = Location + CollisionRadius * X -  40 * Y + 0.4 * CollisionHeight * Z;
	Proj = spawn(class 'DMT_BigRock' ,self,'',projStart,AdjustAim(1000, projStart, 400, true, true));
	if( Proj != None )
		Proj.SetPhysics(PHYS_Projectile);

	if (FRand() < 0.2 * skill)
	{
		projStart = Location + CollisionRadius * X + 40 * Y + 0.4 * CollisionHeight * Z;
		Proj = spawn(class 'DMT_BigRock' ,self,'',projStart,AdjustAim(1000, projStart, 2000, false, true));
		if( Proj != None )
			Proj.SetPhysics(PHYS_Projectile);
	}
}

// This is based upon Botpack.Bot
state Dying
{
ignores SeePlayer, EnemyNotVisible, HearNoise, Died, Bump, Trigger, HitWall, HeadZoneChange, FootZoneChange, ZoneChange, Falling, WarnTarget, LongFall, SetFall, PainTimer;

	function ReStartPlayer()
	{
		if( bHidden && RestartTitan(self) )
		{
			//if ( bNovice )
			//	bDumbDown = ( FRand() < 0.5 );
			//else
			//	bDumbDown = ( FRand() < 0.35 );
			Velocity = vect(0,0,0);
			Acceleration = vect(0,0,0);
			ViewRotation = Rotation;
			//ReSetSkill();
			SetPhysics(PHYS_Falling);
		//SetOrders(BotReplicationInfo(PlayerReplicationInfo).RealOrders, BotReplicationInfo(PlayerReplicationInfo).RealOrderGiver, true);
//			GotoState('Roaming');
			GotoState('Waiting','TurnFromWall');
/*			if (startPoint != None)
				if (TeamGamePlus(Level.Game) != None)
					TeamGamePlus(Level.Game).ChangeTeam(Self,startPoint.Team);
*/
		}
		else if ( !IsInState('GameEnded') )
			GotoState('Dying', 'TryAgain');
	}

Begin:
	if ( Level.Game.bGameEnded )
		GotoState('GameEnded');
	Sleep(0.2);

	if(!bRespawn)
		GotoState('Destroying');

	if ( !bHidden )
		SpawnCarcass();
TryAgain:
	if ( !bHidden )
		HidePlayer();
	Sleep(RespawnDelay);
	ReStartPlayer();
	Goto('TryAgain');
WaitingForStart:
	bHidden = true;
}

// This is identical to Pawn::Dying
State Destroying
{
ignores SeePlayer, EnemyNotVisible, HearNoise, KilledBy, Trigger, Bump, HitWall, HeadZoneChange, FootZoneChange, ZoneChange, Falling, WarnTarget, Died, LongFall, PainTimer;

	function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation, 
							Vector momentum, name damageType)
	{
		if ( bDeleteMe )
			return;
		Health = Health - Damage;
		Momentum = Momentum/Mass;
		AddVelocity( momentum ); 
		if ( !bHidden && Gibbed(damageType) )
		{
			bHidden = true;
			SpawnGibbedCarcass();
			if ( bIsPlayer )
				HidePlayer();
			else
				Destroy();
		}
	}

	function Timer()
	{
		if ( !bHidden )
		{
			bHidden = true;
			SpawnCarcass();
			if ( bIsPlayer )
				HidePlayer();
			else
				Destroy();
		}
	}

	event Landed(vector HitNormal)
	{
		SetPhysics(PHYS_None);
	}

	function BeginState()
	{
		SetTimer(0.3, false);
	}
}

/*
// This state is entered almost every time we kill something...only doesn't happen if we are 
// very busy.
state VictoryDance
{
	function BeginState()
	{
		local string pawnName;
		local Pawn pawnTarget;

		// Actor.Target is an Actor. Şi ai Target ????
		if ( Target != None )
		{
			if(!Target.IsA('Pawn'))
			{
				Log("DMT: VictoryDance: Target is not a pawn.");
				return;
			}
			else
				SayWithPawn(VictoryMsgs[Rand(ArrayCount(VictoryMsgs))], Pawn(Target));
		}
		// Be sure to call the base class implementation.
		//! This MAY have to come before to alleviate the thing where you get hit with 
		// a rock but you don't die!  Seems to work, though didn't test much.
		Super.BeginState();
	}
}
*/

state GameEnded
{
	ignores SeePlayer, HearNoise, KilledBy, Bump, HitWall, HeadZoneChange, FootZoneChange, ZoneChange, Falling, TakeDamage, WarnTarget, Died;

	function BeginState()
	{
		local PlayerPawn pp;
		local UnrealScoreBoard usb;
		local String strWinner;

		// Get rid of our message delayer.
		// Can only use SayCore after the game has ended.
		msgDelayer.bDisabled = true;

		if(Health >= 0)
			SayCore(EndGameAliveMsg);
		else
			SayCore(EndGameDeadMsg);
		
		// *** From Pawn::GameEnded
		//*** HidePlayer();
		SetPhysics(PHYS_None);
		// *** 
		GotoState('EndDancing', 'Begin');
	}
}

function bool PlayDance()
{
	local float decision;

	if (bEndFootStep)
		FootStep();
	DesiredSpeed = 0.0;
	decision = FRand();

	if ( decision < 0.2 )
		PlayAnim('TStom001', 1.0, 0.2);
	else if ( decision < 0.4 )
		PlayAnim('TFist', 1.0, 0.2);
	else if ( decision < 0.64 )
		PlayAnim('TFigh001', 1.0, 0.2);
	else if ( decision < 0.75 )
	{
		PlaySound(Chest, SLOT_Interact);
		PlaySound(Chest, SLOT_Misc);
		PlayAnim('TChest', 1.0, 0.2);
	}
	else
	{
		PlayAnim('TShuffle',1.0, 0.2);
		return true;
	}

	return false;
}

state EndDancing
{
ignores /*SeePlayer, HearNoise,*/ KilledBy, Bump, HitWall, HeadZoneChange, FootZoneChange, ZoneChange, Falling, TakeDamage, WarnTarget, Died;

Begin:
	// Pause a little.
	Sleep(2 + Rand(2));

	// Dance.
	if(PlayDance())
	{
		// If PlayChallenge returns true, then the titan did something like 
		// a "dance" animation, so say a dance message.
		SayCore(DanceMsgs[Rand(ArrayCount(DanceMsgs))]);
	}
	FinishAnim();

	// Do it again.
	Goto 'Begin';
}

// Use this function to initialize our inventory.  Because of a bug in TakeDamage,
// we need to have a not-empty inventory in order for team-damage reduction to work.

/* In Pawn::TakeDamage
...
	if ( bIsPlayer )
	{
		if (ReducedDamageType == 'All') //God mode
			actualDamage = 0;
		else if (Inventory != None) //then check if carrying armor
			actualDamage = Inventory.ReduceDamage(actualDamage, DamageType, HitLocation);
		else
			actualDamage = Damage;
	}
...

See the bug where if the pawn has no inventory the reduced damage isn't used?
This means that we must give the titan an inventory item...
So, give titans a Translator, which should work but not do anything.
*/
function SetInventory()
{
	local Translator s;

	//spawn a searchlight
	if ( FindInventoryType(class'Translator') != None )
		return;
	s = Spawn(class'Translator',,, Location);
	if (s != None)
	{
		s.bHeldItem = true;
		s.GiveTo( self );
		//s.Activate();
		//aPlayer.SelectedItem = s;
	}

	if(inventory==None)
		Log(class$":SetInventory : No damn inventory!");
}

// I found that you have to pop a Titan pretty hard to make him 
// show it.  This gets called typcially when he gets hit when 
// he is close to death.
function PlayHitAnim(vector HitLocation, float Damage)
{
	Super.PlayHitAnim(HitLocation, Damage);

	SayWithPawn(HurtMsgs[Rand(ArrayCount(HurtMsgs))], None);

	//*** Instigator ends up being himself.
	//Log("Playing hit animation, instigator is " $ String(Instigator));
	//Log("Instigator's name is " $ Instigator.PlayerReplicationInfo.PlayerName);
} 

// The TakeDamage methods have been overloaded in both of the following states to
// order to make him talk when someone pops him one when he isn't looking.
state Waiting
{
	function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, name damageType)
	{
		Super.TakeDamage(Damage, instigatedBy, hitLocation, momentum, damageType);
		SayWithPawn(HurtMsgs[Rand(ArrayCount(HurtMsgs))], instigatedBy);
	}
}

state Wandering
{
	ignores EnemyNotVisible;

	function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, name damageType)
	{
		Super.TakeDamage(Damage, instigatedBy, hitLocation, momentum, damageType);
		SayWithPawn(HurtMsgs[Rand(ArrayCount(HurtMsgs))], instigatedBy);
	}
}

// *** Scripted Pawn attitude functions:

// This massive function was copied directly from ScriptedPawn, with the addition that
// Bots have been added to his list of things to hate on sight.
// For UT version, Bot classes also added to list.

function bool SetEnemy( Pawn NewEnemy )
{
	local bool result;
	local eAttitude newAttitude, oldAttitude;
	local bool noOldEnemy;
	local float newStrength;

	if ( NewEnemy.bDeleteMe || NewEnemy == Self || NewEnemy == None || ( NewEnemy != None && NewEnemy.Health <=0 ))
		return false;

//	Log(class$":SetEnemy : My Team = "$PlayerReplicationinfo.Team);
//	Log(class$":SetEnemy : Checking "$NewEnemy$".  Team = "$NewEnemy.PlayerReplicationinfo.Team);

	//*** UT change: if game hasn't started, don't let the titan have enemies.
	/* This should work, but apparently doesn't for CTF games.
	if(level.game.IsA('DeathMatchPlus'))
		if(DeathMatchPlus(level.game).ElapsedTime == 0)
		{
			Log(class$":SetEnemy : I saw someone but ElapsedTime = 0, so I'm ignoring them.");
			return false;
		}
	*/

	// This is a decent workaround for the above: as long as enemy titans don't start off 
	// seeing eachother.
	if(NewEnemy.GetStateName() == 'PlayerWaiting')
	{
		//Log(class$":SetEnemy : A player waiting to join the game just flew by.");
		return false;
	}

	// *** Team change: if we are on a team, make sure we don't attack our enemy.
	if ( NewEnemy.PlayerReplicationInfo != None )
	if( PlayerReplicationInfo.Team ~= NewEnemy.PlayerReplicationInfo.Team )
	{
		//Log(class$":SetEnemy : On my team, not setting them as my enemy.");
		return false;
	}
/*
	Pure SHIT
	if ( !bCanWalk && !bCanFly && !NewEnemy.FootRegion.Zone.bWaterZone )
		return false;
*/
	//*** This is the only line I changed!
	// Old Line:

/*	// New Line:
	if ( (PlayerPawn(NewEnemy) == None) && 
		(ScriptedPawn(NewEnemy) == None) && 
		(Bots(NewEnemy) == None) &&
		(Bot(NewEnemy) == None))
		return false;
*/
	noOldEnemy = (Enemy == None);
	result = false;
	newAttitude = AttitudeTo(NewEnemy);
	//log ("Attitude to potential enemy is "$newAttitude);
	if ( !noOldEnemy )
	{
		if (Enemy == NewEnemy)
			return true;
		else if ( NewEnemy.bIsPlayer && (AlarmTag != '') )
		{
			OldEnemy = Enemy;
			Enemy = NewEnemy;
			result = true;
		} 
/*		else if ( newAttitude == ATTITUDE_Friendly )
		{
//			if ( bIgnoreFriends )
//				return false;
			if ( (NewEnemy.Enemy != None) && (NewEnemy.Enemy.Health > 0) ) 
			{
				if ( NewEnemy.Enemy.bIsPlayer && (NewEnemy.AttitudeToPlayer < AttitudeToPlayer) )
					AttitudeToPlayer = NewEnemy.AttitudeToPlayer;
				if ( AttitudeTo(NewEnemy.Enemy) < AttitudeTo(Enemy) )
				{
					OldEnemy = Enemy;
					Enemy = NewEnemy.Enemy;
					result = true;
				}
			}
		}
*/
		else 
		{
			oldAttitude = AttitudeTo(Enemy);
			if ( (newAttitude < oldAttitude) || 
				( (newAttitude == oldAttitude) 
					&& ((VSize(NewEnemy.Location - Location) < VSize(Enemy.Location - Location)) 
						|| !LineOfSightTo(Enemy)) ) ) 
			{
				if ( bIsPlayer && Enemy.IsA('PlayerPawn') && !NewEnemy.IsA('PlayerPawn') )
				{
					newStrength = relativeStrength(NewEnemy);
					if ( (newStrength < 0.2) && (relativeStrength(Enemy) < FMin(0, newStrength))  
						&& (IsInState('Hunting')) && (Level.TimeSeconds - HuntStartTime < 5) )
						result = false;
					else
					{
						result = true;
						OldEnemy = Enemy;
						Enemy = NewEnemy;
					}
				} 
				else
				{
					result = true;
					OldEnemy = Enemy;
					Enemy = NewEnemy;
				}
			}
		}
	}
	else if ( newAttitude < ATTITUDE_Ignore )
	{
		result = true;
		Enemy = NewEnemy;
	}
	if ( Hated != None && Hated == NewEnemy )
	{
		Enemy = Hated;
		result = True;
	}
	else if ( newAttitude == ATTITUDE_Friendly ) //your enemy is my enemy
	{
		//log("noticed a friend");
/*		if ( NewEnemy.bIsPlayer && (AlarmTag != '') )
		{
			Enemy = NewEnemy;
			result = true;
		} 
		if (bIgnoreFriends)
			return false;
*/
		if ( (NewEnemy.Enemy != None) && (NewEnemy.Enemy.Health > 0) ) 
		{
			//log("his enemy is my enemy");
			Enemy = NewEnemy.Enemy;
			result = true;
/*			if (Enemy.bIsPlayer)
				AttitudeToPlayer = ScriptedPawn(NewEnemy).AttitudeToPlayer;
			else if ( (ScriptedPawn(NewEnemy) != None) && (ScriptedPawn(NewEnemy).Hated == Enemy) )
				Hated = Enemy;
*/
		}
	}

	if ( result )
	{
		//log(class$" has new enemy - "$enemy.class);
		LastSeenPos = Enemy.Location;
		LastSeeingPos = Location;
		EnemyAcquired();
		if ( !bFirstHatePlayer && Enemy.bIsPlayer && (FirstHatePlayerEvent != '') )
			TriggerFirstHate();
	}
	else if ( NewEnemy.bIsPlayer && (NewAttitude < ATTITUDE_Threaten) )
		OldEnemy = NewEnemy;
	return result;
}

/* AttitudeTo()
Returns the creature's attitude towards another Pawn
*/
// ScriptedPawn version.
// Ns > Anyone heard about bTeamGame ? Nobody ? "Unfortunately" I'll fix these stupid craps.

function eAttitude AttitudeTo(Pawn Other)
{
	if ( Other == Self || Other == None || Other.Mesh == None || Other.bHidden ) //added
		return ATTITUDE_Friendly;
	else
	{
// *** Check for teammates. (Has to be done before same class check!)
// *** No! It's bullshit...
		if ( Other.PlayerReplicationInfo != None )
		{
			if ((Level.Game).bTeamGame )
			{
				if( PlayerReplicationInfo.Team ~= Other.PlayerReplicationInfo.Team)
					return ATTITUDE_Friendly;
				else
					return ATTITUDE_Hate;
			}
			else
				return ATTITUDE_Hate;
			//*** See if we are to hate our own type of beast.
			//*** Ns > Shut up! Did you see that is shit what you had here?
			//*** Now is fixed, :P
		}
// This attitude-soup is amazing. You could think twice
//		else if ( bHateOurOwn && Other.class == class )
//		{
//			//Log(class$":AttitudeTo : Hating since their class is same as ours.");
//			return ATTITUDE_Hate;
//		}

		else
		{
			if ((Level.Game).bTeamGame)
			{
				if ( ScriptedPawn(Other) != None )
				{
					if ( TeamTag != '' && ScriptedPawn(Other).TeamTag == TeamTag )
						return ATTITUDE_Friendly;
					else if ( ScriptedPawn(Other).Team ~= PlayerReplicationInfo.Team )
						return ATTITUDE_Ignore; //Being player will be attacked anyway so will respond properly according to master Attitude
					else
						return ATTITUDE_Hate;
				}
				else
					return ATTITUDE_Hate;
			}
			else
				return ATTITUDE_Hate;
		}
//	return Super.AttitudeTo(Other);
	}
}

function BotVoiceMessage(name messagetype, byte MessageID, Pawn Sender)
{
	// Respond when someone tries to give us an order.
	if ( messagetype == 'ORDER' )
	{
		// Automatically prepend an "@" to make it a team-only msg.
		SayWithPawn("@"$IgnoreOrderMsgs[Rand(ArrayCount(IgnoreOrderMsgs))], Sender);
	}
}
// Fixing original mess

function bool MeleeDamageTarget(int hitdamage, vector pushdir)
{
	local vector HitLocation, HitNormal, TargetPoint;
	local actor HitActor;

	if ( ( Enemy != None && Enemy != Self ) && Target == None )
		Target = Enemy;
	if ( Target != None && Target.bCollideActors )
	{
		If ( (VSize(Target.Location - Location) <= MeleeRange * 1.4 + Target.CollisionRadius + CollisionRadius)
			&& ((Physics == PHYS_Flying) || (Physics == PHYS_Swimming) || (Abs(Location.Z - Target.Location.Z) 
				<= FMax(CollisionHeight, Target.CollisionHeight) + 0.5 * FMin(CollisionHeight, Target.CollisionHeight))) )
		{
			HitActor = Trace(HitLocation, HitNormal, Target.Location, Location, false);
			if ( HitActor != None )
				return false;
			Target.TakeDamage(hitdamage, Self,HitLocation, pushdir, 'hacked');
			return true;
		}
	}
	return false;
}

function PunchDamageTarget()
{
	if (Target != None && Target != Self )
	{
		if ( MeleeDamageTarget(PunchDamage, (70000.0 * (Normal(Target.Location - Location)))) )
		{
			PlaySound(Slap, SLOT_Interact);
			PlaySound(Slap, SLOT_Misc);
		}
	}
}

function SlapDamageTarget()
{
	local vector X,Y,Z;
	GetAxes(Rotation,X,Y,Z);

	if (Target != None && Target != Self )
	{
		if ( MeleeDamageTarget(SlapDamage, (70000.0 * ( Y + vect(0,0,1)))) )
		{
			PlaySound(Slap, SLOT_Interact);
			PlaySound(Slap, SLOT_Misc);
		}
	}
}

function SeePlayer(Actor SeenPlayer)
{
	//log(class$" has line of sight to player");
	if ( SeenPlayer != None )
	{
		if ( SeenPlayer == Self || !SeenPlayer.bCollideActors || SeenPlayer.bHidden )
			return;
		else
		{
			if ( SetEnemy(Pawn(SeenPlayer)) )
				LastSeenPos = Enemy.Location;
		}
	}
}

function Bump(actor Other)
{
	local vector VelDir, OtherDir;
	local float speed;

	if (Other == Self)
		return;
	else if ( Enemy != None )
	{
		if ( Other == Enemy )
		{
			Target = Other;
			GotoState('MeleeAttack');
			return;
		}
		else if ( (Pawn(Other) != None) && SetEnemy(Pawn(Other)) )
		{
			Target = Other;
			GotoState('MeleeAttack');
			return;
		} 
	}
	else
	{
		if (Pawn(Other) != None)
		{
			AnnoyedBy(Pawn(Other));
			if ( SetEnemy(Pawn(Other)) )
			{
				bReadyToAttack = True; //can melee right away
				PlayAcquisitionSound();
				GotoState('Attacking');
				return;
			}
		}
		if ( TimerRate <= 0 )
			setTimer(1.0, false);
		if ( bCanSpeak && (ScriptedPawn(Other) != None) && ((TeamLeader == None) || !TeamLeader.bTeamSpeaking) )
			SpeakTo(ScriptedPawn(Other));
	}

	speed = VSize(Velocity);
	if ( speed > 1 )
	{
		VelDir = Velocity/speed;
		VelDir.Z = 0;
		OtherDir = Other.Location - Location;
		OtherDir.Z = 0;
		OtherDir = Normal(OtherDir);
		if ( (VelDir Dot OtherDir) > 0.8 )
		{
			Velocity.X = VelDir.Y;
			Velocity.Y = -1 * VelDir.X;
			Velocity *= FMax(speed, 280);
		}
	} 
	Disable('Bump');
}

state Attacking
{
ignores SeePlayer, HearNoise, Bump, HitWall;

	function ChooseAttackMode()
	{
		local eAttitude AttitudeToEnemy;
		local float Aggression;
		local pawn changeEn;

		if (Enemy == Self || Enemy == None || Enemy.Health <= 0 || Enemy.Mesh == None )
		{
			if (Orders == 'Attacking')
				Orders = '';
			WhatToDoNext('','');
			return;
		}

		if ( (AlarmTag != '') && Enemy.bIsPlayer )
		{
			if (AttitudeToPlayer > ATTITUDE_Ignore)
			{
				GotoState('AlarmPaused', 'WaitForPlayer');
				return;
			}
			else if ( (AttitudeToPlayer != ATTITUDE_Fear) || bInitialFear )
			{
				GotoState('TriggerAlarm');
				return;
			}
		}

		AttitudeToEnemy = AttitudeTo(Enemy);

		if (AttitudeToEnemy == ATTITUDE_Fear)
		{
			GotoState('Retreating');
			return;
		}

		else if (AttitudeToEnemy == ATTITUDE_Threaten)
		{
			GotoState('Threatening');
			return;
		}

		else if (AttitudeToEnemy == ATTITUDE_Friendly)
		{
			if (Enemy.bIsPlayer)
				GotoState('Greeting');
			else
				WhatToDoNext('','');
			return;
		}

		else if (!LineOfSightTo(Enemy))
		{
			if ( (OldEnemy != None) 
				&& (AttitudeTo(OldEnemy) == ATTITUDE_Hate) && LineOfSightTo(OldEnemy) )
			{
				changeEn = enemy;
				enemy = oldenemy;
				oldenemy = changeEn;
			}
			else 
			{
				if ( (Orders == 'Guarding') && !LineOfSightTo(OrderObject) )
					GotoState('Guarding');
				else if ( !bHasRangedAttack || VSize(Enemy.Location - Location) 
							> 600 + (FRand() * RelativeStrength(Enemy) - CombatStyle) * 600 )
					GotoState('Hunting');
				else if ( bIsBoss || (Intelligence > BRAINS_None) )
				{
					HuntStartTime = Level.TimeSeconds;
					NumHuntPaths = 0; 
					GotoState('StakeOut');
				}
				else
					WhatToDoNext('Waiting', 'TurnFromWall');
				return;
			}
		}

		else if ( (TeamLeader != None) && TeamLeader.ChooseTeamAttackFor(self) )
			return;

		if (bReadyToAttack)
		{
			////log("Attack!");
			Target = Enemy;
			If (VSize(Enemy.Location - Location) <= (MeleeRange + Enemy.CollisionRadius + CollisionRadius))
			{
				GotoState('MeleeAttack');
				return;
			}
			else if (bMovingRangedAttack)
				SetTimer(TimeBetweenAttacks, False);
			else if (bHasRangedAttack && (bIsPlayer || enemy.bIsPlayer) && CanFireAtEnemy() )
			{
				if (!bIsPlayer || (2.5 * FRand() > Skill) )
				{
					GotoState('RangedAttack');
					return;
				}
			}
		}

		//decide whether to charge or make a tactical move
		if ( !bHasRangedAttack ) 
			GotoState('Charging');
		else
			GotoState('TacticalMove');
		//log("Next state is "$state);
	}

	//EnemyNotVisible implemented so engine will update LastSeenPos
	function EnemyNotVisible()
	{
		////log("enemy not visible");
	}

	function Timer()
	{
		bReadyToAttack = True;
	}

	function BeginState()
	{
		if ( TimerRate <= 0.0 )
			SetTimer(TimeBetweenAttacks  * (1.0 + FRand()),false); 
		if (Physics == PHYS_None)
			SetMovementPhysics(); 
	}

Begin:
	//log(class$" choose Attack");
	ChooseAttackMode();
}

state VictoryDance
{
ignores EnemyNotVisible; 

	function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation, 
							Vector momentum, name damageType)
	{
		Global.TakeDamage(Damage, instigatedBy, hitlocation, momentum, damageType);
		if ( health <= 0 )
			return;
		Enemy = instigatedBy;
		if ( NextState == 'TakeHit' )
		{
			NextState = 'Attacking'; //default
			NextLabel = 'Begin';
			GotoState('TakeHit'); 
		}
		else if (health > 0)
			GotoState('Attacking');
	}

	function EnemyAcquired()
	{
		//log(Class$" just acquired an enemy");
		GotoState('Acquisition');
	}

	function PickDestination()
	{
		local Actor path;
		local vector destpoint;

		if (Target == None)
		{
			WhatToDoNext('Waiting', 'TurnFromWall'); 
			return;
		}
		destpoint = Target.Location;
		destpoint.Z += CollisionHeight - Target.CollisionHeight;
		if (pointReachable(destpoint))
		{
			MoveTarget = Target;
			Destination = destpoint;
		}
		else
		{
			if (SpecialGoal != None)
				path = FindPathToward(SpecialGoal);
			else
				path = FindPathToward(Target);
			if (path != None)
			{
				MoveTarget = path;
				Destination = path.Location;
			}
			else
				WhatToDoNext('Waiting', 'TurnFromWall'); 
		}
	}

	function BeginState()
	{
		SpecialGoal = None;
		SpecialPause = 0.0;
		SetAlertness(-0.3);
	}

Begin:
	if ( Target == None)
		PickDestination();
	else
	{
	if (VSize(Location - Target.Location) < 
		(1.3 * CollisionRadius + Target.CollisionRadius + CollisionHeight - Target.CollisionHeight))
			Goto('Taunt');
	Destination = Target.Location;
	}
	TweenToWalking(0.3);
	FinishAnim();
	PlayWalking();
	Enable('Bump');

MoveToEnemy:

	WaitForLanding();
	PickDestination();
	if (SpecialPause > 0.0)
	{
		Acceleration = vect(0,0,0);
		TweenToPatrolStop(0.3);
		Sleep(SpecialPause);
		SpecialPause = 0.0;
		TweenToWalking(0.1);
		FinishAnim();
		PlayWalking();
	}
	MoveToward(MoveTarget, WalkingSpeed);
	Enable('Bump');
	if ( Target != None )
		if (Target != None && VSize(Location - Target.Location) < 
		(1.3 * CollisionRadius + Target.CollisionRadius + Abs(CollisionHeight - Target.CollisionHeight)))
		Goto('Taunt');
	Goto('MoveToEnemy');

Taunt:
	Acceleration = vect(0,0,0);
	TweenToFighter(0.2);
	FinishAnim();
	PlayTurning();
	if (Target != None)
		TurnToward(Target);
	DesiredRotation = rot(0,0,0);
	DesiredRotation.Yaw = Rotation.Yaw;
	setRotation(DesiredRotation);
	TweenToFighter(0.2);
	FinishAnim();
	PlayVictoryDance();
	FinishAnim(); 
	WhatToDoNext('Waiting','TurnFromWall');
}

// End of fixing original mess
defaultproperties
{
     MyName="Mr.Titan"
     DefaultName="Titan"
     Team=3
     bRespawn=True
     RespawnDelay=20.000000
     bHateOurOwn=True
     fDelayBaseSecs=0.700000
     fDelaySecsPerChar=0.150000
     fWaitMsgProbability=0.100000
     TriggerMsgs(0)="Hey %p come back here!"
     TriggerMsgs(1)="%p is running like a little homo."
     TriggerMsgs(2)="Hey %p why don't you try that again?"
     TriggerMsgs(3)="Come back down here %p - I dare you!"
     TriggerMsgs(4)="I triple-dog-dare you to come back down here."
     TriggerMsgs(5)="You won't get away next time!"
     TriggerMsgs(6)="Shit!"
     TriggerMsgs(7)="You little pussy."
     TriggerMsgs(8)="%p is a little camping bitch."
     TriggerMsgs(9)="Come back here pussy."
     TriggerMsgs(10)="None of you faggots will fight me =("
     TriggerMsgs(11)="Fuckin - A =("
     TriggerMsgs(12)="Come back here %p. I won't be mean"
     TriggerMsgs(13)="=("
     TriggerMsgs(14)="Heh, you won't get away next time %p."
     TriggerMsgs(15)="Grrr."
     TriggerMsgs(16)="You won't get away next time!"
     TriggerMsgs(17)="Shit!"
     TriggerMsgs(18)="You little pussy."
     TriggerMsgs(19)="Why you running %p?"
     TriggerMsgs(20)="Hey %p try coming down here again."
     TriggerMsgs(21)="Come back here asshole."
     TriggerMsgs(22)="Teleporting is so pussy-ish."
     TriggerMsgs(23)="Yeah %p is running away like a little queer."
     TriggerMsgs(24)="Hey %p, you'll learn your fuckin lesson next time."
     TriggerMsgs(25)="You old pussy-ass."
     TriggerMsgs(26)="Ya'll come back now y'hear?"
     TriggerMsgs(27)="%p is such a wussy."
     TriggerMsgs(28)="Come back here %p."
     TriggerMsgs(29)="Hang around next time %p so I can school you."
     TriggerMsgs(30)="Come back %p, I won't bite."
     DyingMsgs(0)="%p got lucky!"
     DyingMsgs(1)="%p, you're an asshole!"
     DyingMsgs(2)=":-("
     DyingMsgs(3)="Shit, %p got me!!!"
     DyingMsgs(4)="I'm dead, you all still suck though!"
     DyingMsgs(5)="I didn't feel like playing anymore anyway!"
     DyingMsgs(6)="%p killed me, but you still suck."
     DyingMsgs(7)="=("
     DyingMsgs(8)="Dammit that hurt!!!"
     DyingMsgs(9)="Oh no you killed me, you bastards!!!"
     DyingMsgs(10)="I'll be back."
     DyingMsgs(11)="Hey %p, I demand a rematch."
     DyingMsgs(12)="Fuck that =("
     DyingMsgs(13)="Damn"
     DyingMsgs(14)="heh...I let you win %p"
     DyingMsgs(15)="Fuck that, %p dies next time."
     DyingMsgs(16)="Man fuck this shit."
     DyingMsgs(17)="I just let %p kill me cause I'm tired of dominating."
     DyingMsgs(18)="I was cleaning my mouse, that's why I died."
     DyingMsgs(19)="You shot me while I was typing you faggot."
     DyingMsgs(20)="Ouch =("
     VictoryMsgs(0)="Hey %p. You just got knocked the fuck out!"
     VictoryMsgs(1)="%p is such a pussy."
     VictoryMsgs(2)="My grandma hits harder than %p, and she's dead!"
     VictoryMsgs(3)="%p really sucks, I must say."
     VictoryMsgs(4)="Hey %p - that's your first lesson in Beat-Down101."
     VictoryMsgs(5)="I just kicked your ass %p.  "
     VictoryMsgs(6)="Hehe!"
     VictoryMsgs(7)="Lol!"
     VictoryMsgs(8)="%p sucks!"
     VictoryMsgs(9)="Hey %p this your first time playing???"
     VictoryMsgs(10)="Doh!!"
     VictoryMsgs(11)="Hey %p you suck bad!!!"
     VictoryMsgs(12)="I rule!!!"
     VictoryMsgs(13)="I win, you lose, any questions?"
     VictoryMsgs(14)=":-)"
     VictoryMsgs(15)="%p - you just got your ass kicked and that's the bottom fuckin line."
     VictoryMsgs(16)="Try again %p."
     VictoryMsgs(17)="You picked the wrong day to fuck with me!"
     VictoryMsgs(18)="Who's your daddy?"
     VictoryMsgs(19)="%p - come back soon for another beat down!"
     VictoryMsgs(20)="How many of you fuckin numb-nuts must I kill?"
     VictoryMsgs(21)="I feel bad killing newbies like %p."
     VictoryMsgs(22)="Hey %p. Why don't you try again when you how to play this fuckin game?"
     VictoryMsgs(23)="Goddam, I just beat the shit out of %p"
     VictoryMsgs(25)="Haha %p you suck =P"
     VictoryMsgs(26)="=P"
     VictoryMsgs(27)="Hehe =P"
     VictoryMsgs(28)="%p just got his ass kicked =)"
     VictoryMsgs(29)="Ok %p just learned his fuckin lesson...who's next?"
     VictoryMsgs(30)="Lol! The 'competition' here is pathetic."
     VictoryMsgs(31)="No match."
     VictoryMsgs(32)="Hehe. I just kicked the shit out of %p."
     VictoryMsgs(33)="%p is a hell of a lot skinnier now."
     VictoryMsgs(34)="Hey %p, how many times before you learn your fucking lesson?"
     VictoryMsgs(35)="Who's your daddy %p? =P"
     VictoryMsgs(36)="I'm sorry %p, did you want that arm?"
     VictoryMsgs(37)="%p just got fuckin owned."
     VictoryMsgs(38)="Looks like %p had to learn the hard way."
     VictoryMsgs(39)="Loser."
     VictoryMsgs(40)="%p just got bitch slapped."
     VictoryMsgs(41)="I slapped the shit out of %p. hehe."
     VictoryMsgs(42)="Damn the emergency room is gonna be busy tonight."
     VictoryMsgs(43)="Hey %p, I think that's gonna leave a mark."
     VictoryMsgs(44)="%p has no skills. hehe."
     VictoryMsgs(45)="Lol!!!!!!!!!"
     VictoryMsgs(46)="heh =P"
     VictoryMsgs(47)="%p just got schooled."
     VictoryMsgs(48)="%p is a pathetic fuckin loser."
     VictoryMsgs(49)="I am the fuckin man."
     VictoryMsgs(50)="Damn %p, you suck at UT."
     HurtMsgs(0)="Ouch!"
     HurtMsgs(1)="You fuckin queer."
     HurtMsgs(2)="That supposed to hurt?"
     HurtMsgs(3)="You hit like a bitch %p."
     HurtMsgs(4)="I'm gonna shove that gun up your ass."
     HurtMsgs(5)="Ow!"
     HurtMsgs(6)="Stop that!"
     HurtMsgs(7)="This is my tent bitch."
     HurtMsgs(8)="Just wondering, is that supposed to hurt?"
     HurtMsgs(9)="Your toy gun is no match for me fucker."
     HurtMsgs(10)="Ouchies."
     HurtMsgs(11)="Quit shootin me %p. Make it easier on yourself"
     HurtMsgs(12)="%p just pissed me the fuck off."
     HurtMsgs(13)="Owie =("
     HurtMsgs(14)="Hey quit it, that's gonna leave a scar."
     HurtMsgs(15)="Grrr!"
     HurtMsgs(16)="You just insured your own death %p."
     HurtMsgs(17)="%p is about to get a case of whoop ass."
     HurtMsgs(18)="Hey that hurt...quit it."
     HurtMsgs(19)="I'm gonna shove that gun up your ass sideways."
     HurtMsgs(21)="Stop shooting me douche bag."
     HurtMsgs(22)="Quit it fuckface."
     HurtMsgs(23)="Haha real funny %p, time to die."
     HurtMsgs(24)="You just signed your own death warrant %p."
     HurtMsgs(25)="Hehe, that's supposed to hurt?"
     HurtMsgs(26)="Wow you suck %p."
     HurtMsgs(27)="Nice try %p, now let me show you how it's done."
     HurtMsgs(28)="Ouch! Time to die %p."
     HurtMsgs(29)="=("
     HurtMsgs(30)="Time to fruitcake."
     WaitMsgs(0)="You all suck! Come get me!"
     WaitMsgs(1)="You all suck bad!"
     WaitMsgs(2)="Why don't one of you pussies come down here?"
     WaitMsgs(3)="Come on down the water is fine!"
     WaitMsgs(4)="This is boring."
     WaitMsgs(5)="Check out my ping fuckers!"
     WaitMsgs(6)="I could beat all of you with one hand tied to my nuts."
     WaitMsgs(7)="This would be more fun if you weren't a bunch of fuckin campers."
     WaitMsgs(8)="I've had tougher matches against a girl scout troop."
     WaitMsgs(9)="Killing you fags is so easy it's pathetic."
     WaitMsgs(10)="I've had tougher matches against a pregnant Nali."
     WaitMsgs(11)="You're all fuckin pathetic."
     WaitMsgs(12)="=/"
     WaitMsgs(13)="Lalala...killing you losers is getting boring."
     WaitMsgs(14)="This would be better if you weren't a bunch of fuckin newbies."
     WaitMsgs(15)="The only winning move you queers can make is to leave the game."
     WaitMsgs(16)="I rule."
     WaitMsgs(17)="Someone come here, I need another frag."
     WaitMsgs(18)="I've had more of a challenge against a dead squirrel."
     WaitMsgs(19)="You fags are no match, I'm gonna go play Q3."
     WaitMsgs(20)="This is more like romper room than freakin deathmatch."
     WaitMsgs(21)="You're just jealous."
     WaitMsgs(22)="Anyone got the balls to come down here?"
     WaitMsgs(23)="How bout one of you fuckin numb nuts come down here?"
     WaitMsgs(24)="Killing you queers is real hard.....not."
     WaitMsgs(25)="You all picked the wrong day to fuck with me."
     WaitMsgs(26)="I have a PHD in assholism."
     WaitMsgs(27)="I'm teaching a class in how to get slapped the fuck down."
     WaitMsgs(28)="Ok who wants to fuck with me?"
     WaitMsgs(29)="Who's next?"
     WaitMsgs(30)="How many numb-nuts does it fuckin take to kill a Titan?"
     WaitMsgs(31)="You're a bunch of pansies."
     WaitMsgs(32)="Obviously you all suck at UT. How bout some Starcraft?"
     WaitMsgs(33)="I'm the fuckin man."
     WaitMsgs(34)="You all need to go back the fuckin N-64."
     WaitMsgs(35)="I've never seen a bigger collection of losers in my life."
     WaitMsgs(36)="Goddam, you all act like you're in fuckin pre-school."
     WaitMsgs(37)="What a bunch of lamers."
     WaitMsgs(38)="You people have some really sad 'skillz'."
     WaitMsgs(39)="Come on, can anyone put up a fight?"
     WaitMsgs(40)="I have a PHD in handing out fuckin beat downs."
     IgnoreOrderMsgs(0)="I'd rather not."
     IgnoreOrderMsgs(1)="Kiss my ass!"
     IgnoreOrderMsgs(2)="Your face, my ass."
     IgnoreOrderMsgs(3)="I don't take orders from nobody."
     IgnoreOrderMsgs(4)="Uh, I'll just stay here and smash things."
     IgnoreOrderMsgs(5)="I don't take orders from hairless monkeys."
     IgnoreOrderMsgs(6)="I assume these orders don't apply to me."
     IgnoreOrderMsgs(7)="Not in my job description..."
     InsultingNames(0)="punk"
     InsultingNames(1)="sucker"
     InsultingNames(2)="loser"
     InsultingNames(3)="lamer"
     InsultingNames(4)="newbie"
     InsultingNames(5)="asshole"
     InsultingNames(6)="cocksucker"
     InsultingNames(7)="prick"
     InsultingNames(8)="fuckwad"
     InsultingNames(9)="scumbag"
     DanceMsgs(0)="Good God, y'all!"
     DanceMsgs(1)="Get down!"
     DanceMsgs(2)="I can do the boogaloo!"
     DanceMsgs(3)="Watch me now!"
     DanceMsgs(4)="I'm cuttin' a rug!"
     EndGameAliveMsg="gg"
     EndGameDeadMsg="That game sucked!"
     iPingMaxVariation=75
     iPingRefreshBase=4
     iPingRefreshMaxVariation=2
     Health=3500
}
