//=============================================================================
// UTDMTMsgDelayer.
//This class delays messages that come from a UTDMT based on length.  The idea is
//to simulate the amount of time it would take to type in the message.
//-The class will cache up to two messages, but no more.  Caching more may cause
//confusing text if a bunch of messages get generated at once.
//=============================================================================
class UTDMTMsgDelayer extends Actor;

var UTDMT dmtOwner;

var float fBaseSecs;
var float fSecsPerChar;

var string strDelayed;
var string strCached;

var bool bDisabled;

// ***

function DelayMsg(string msg)
{
	local float fDelay;

	if(bDisabled)
	{
		Log(class$":DelayMsg('"$msg$"') called, but we are disabled.");
		return;
	}

	if(strDelayed != "")
	{
		// If there is another message cached we don't care: just overwrite
		// it.  This keeps the cached message as 'current' as possible.
		strCached = msg;
		return;
	}

	strDelayed = msg;
	fDelay = fBaseSecs + (fSecsPerChar * Len(msg));
	SetTimer(fDelay, false);
}

// There doesn't seem to be a way to stop the timer, but if we destroy the strings
// it won't do anything.  I think that if we set a new timer it might override the last
// anyhow.
function Flush()
{
	strCached = "";
	strDelayed = "";
}

event Timer()
{
	local string strTemp;

	if(bDisabled)
	{
		strCached = "";
		strDelayed = "";
		return;
	}

	if(strDelayed != "" && dmtOwner != None )
		dmtOwner.SayCore(strDelayed);
	strDelayed = "";

	if(strCached != "")
	{
		strTemp = strCached;
		strCached = "";
		DelayMsg(strTemp);
	}
}

defaultproperties
{
     fBaseSecs=1.000000
     fSecsPerChar=0.200000
     bHidden=True
}
